/*

######################################################################
######################################################################
##                                                                  ##
##                                                                  ##
##                            statnet                               ##
##                                                                  ##
##            a feedforward interpolation neural network            ##
##                                                                  ##
##                              by                                  ##
##                                                                  ##
##                       Coryn Bailer-Jones                         ##
##                                                                  ##
##                           07/12/98                               ##
##                                                                  ##
##             email: calj@mpia-hd.mpg.de                           ##
##               www: http://www.mpia-hd.mpg.de/stars/calj/         ##
##                                                                  ##
##                                                                  ##
##        see the README file for disclaimer and warranty           ##
##        see the statnet_manual file for operational details       ##
##                                                                  ##
##     This file is copyright 1998,1999 by C.A.L. Bailer-Jones      ##
##                                                                  ##
##                                                                  ##
######################################################################
######################################################################


FILE:		statnet.h
DESCRIPTION:    ANSI C statnet header file
AUTHOR:		Coryn Bailer-Jones
LAST MOD DATE:	24.06.99


######################################################################
######################################################################
##                                                                  ##
##                           statnet.h                              ##
##                                                                  ##
######################################################################
######################################################################

*/

#include "ansi/macopt.h"
#include "netsubs.h"

#define VERSION  "2.00"
#define LASTMODDATE "24.06.99"


/*
---------------------------------------------------------------------
- user should check whether the fixed arrays are sufficiently large -
---------------------------------------------------------------------
*/

/* maximum array sizes - application dependent */

#define MAXRANGES      100   /* maximum number of ranges */
#define MAXOUTS       1000   /* maximum number of outputs
				only one integer array is defined to this
				size */
#define MAXEXCLUDE     100   /* maximum number of patterns to exclude */
#define WORDSIZE       500   /* various strings e.g. from specfile */
#define FNAMESIZE      200   /* any file name */

/* warnings given if the following sizes are exceeded 
   (but it will cause no problems) */

#define COMWARNSIZE     20  /* will warn if number of networks in
			       committee exceeds this value */
#define NODEWARNSIZE  1000  /* will warn if number of nodes in any
			       network layer exceeds this value */
#define WTRNGWARNSIZE    5  /* will warn if initial weight range
			       exceeds this value */

/* recognised file suffices */

#define INFILESUFFIX  "in"  /* input data files ending in this will */
#define OTFILESUFFIX  "ot"  /* have an output file ending in this */


/*
--------------------------------------------------------------------
----- user should have no reason to change anything below here -----
                      DO SO AT YOUR OWN PERIL!
--------------------------------------------------------------------
*/


/* defaults */

#define HIDBIAS          1.0   /* hidden layer bias node value */
#define INPBIAS          1.0   /* input  layer bias node value */

#define NALPHA           4     /* number of alpha parameters */
#define alpha_XH_DEF     0.04  /* defaults for the weight decay terms */
#define alpha_bH_DEF     0.04
#define alpha_HV_DEF     0.04
#define alpha_VY_DEF     0.04
#define BETADEF          6.0   /* default noise level */

#define WTRNGDEF         0.1   /* default range of intial random weights */
#define RANSEEDDEF     731
#define ITSDEF         100     /* default number of iterations for grad descent */
#define ETADEF           0.05  /* default value of eta in gradient descent */
#define MACCONVTOLDEF   10.0
#define MACCHECKTOLDEF   0.000001 /* default macchecktol value */

#define LOGSTEM       "NET:TAKELOG"
#define INRANGESTEM   "DAT:INRANGE"
#define OTRANGESTEM   "DAT:OTRANGE"
#define BETASTEM      "TRN:beta"   
#define DEFWTNAME     "statnet.wt" /* default output weight file name */


/* maximum array sizes - application independent */

#define Nflags         100   /* number of flags */
#define MESSAGESIZE   1000   /* messages */


/* type definitions */

typedef struct {
  /* contains arguments for statnet gradient evaluation function */
  int Xsize;
  int Hsize;
  int Vsize;
  int Ysize;
  double *alpha;
  double *beta;
  double **x;
  targets **tar; 
  double Hlam;
  double Vlam;
  int Npats;
  int *flag;
  FILE *errf;     /* file handle for error file */
  int wtvec_size;
  double *gtyp;   /* pointer to macopt rms gradient */
  int *its;       /* pointer to current macopt iteration number */
  int previts;    /* previous macopt iteration number */
  double lerr;    /* the likelihood contribution to the error */
  double werr;    /* the weight decay contribution to the error */
} statmacint_args;

/* prototypes in statnet.c */

void specread(char *, int *, int *, int *, int *, int *, int *, int *, int *,
	      int **, int **, int *, double *, char *, char *,
	      char *, char *, long *, double *, int *, double *,
	      double *, char *, int *, char **);

void statnettrain(int *, int, int, int, int, int, int, int, int **, int **,
		  double ***, double ***, double ***,
		  double **, double **, double *, double *, char *, int, double,
		  char *, char *, char *, long, double, int, double,
		  double, char *, int *, char *[]);

void statnetapply(int *, int, int, int, int, int, int, int, int **, int **,
		  double ***, double ***, double ***, double **, double **,
		  double *, double *, char *, char *, char *, char *, long, double,
		  int *, char *[]);

void cominit(int, int, int, int, int, double ***, double ***, double ***,
	     double **, double **, double *,  double *, int *, char *, char *, long,
	     double *, double *, double);

void dataread(int, int, int, int, int **, int **, 
	      int *, char *, double ***, targets ***, int *, int *, 
	      char **, int *);

void scalecalc(int, int, int, int, double **, targets **, 
	       double **, double **, double *, double *, int *);

void datascale(int, int, int, double **, targets **, double **, double **, int *);

void unscale(int, double *, double **, int *);

void forpass(int, int, int, double *, double *, double *, 
	     double **, double **, double, int *);

void forpass2(int, int, int, int, 
	      double *, double *, double *, double *,
	      double **, double **, double **, double, double, int *);

void ederiv(int, int, int, double *, double *, double *, 
	    double **, double **, double **, double **, 
	    targets *, double *, double *, double, int *);

void ederiv2(int, int, int, int, double *, double *, double *, double *,
	     double **, double **, double **, 
	     double **, double **, double **, 
	     targets *, double *, double *, double, double, int *);

void cumederivs(int, int, int, int, double **, double **, 
		double **, double **, double **, double **, int *);

void updatewt(int, int, int, int, double **, double **, double **, 
	      double **, double **, double **, double, int *);

void graddescent(int, int, int, int, double **, double **, double **, double **,
		 targets **, int, double *, double *, double, double, int *, int, 
		 double, char *);
	
void callmacopt(int, int, int, int, double **, double **, double **, double **,
		targets **, int, double *, double *, double,  double, int *, 
		char *, double, int, double);

void statmacint(double *, double *, void *);

double statmacfn(double *, void *);

void writeweights(int, int, int, int, int, double ***, double ***, double ***,
		  double **, double **, double, double, char *, int *);

void evnewname(char *, char *);

double wderr(int, int, int, int, double **, double **, double **, double *, int *);



/* external prototypes in their own files */

float ran1(long *); 
